import BN from "bn.js";
import { recover } from "./crypto";
import { secp256k1N, secp256k1halfN } from "./constant";
import { Address } from "./address";
export class Signature {
    constructor(buffer) {
        if (buffer.length !== 65) {
            throw TypeError(`Signature: invalid signature. buffer length must be 65. actual: ${buffer.length}`);
        }
        this.buffer = buffer;
        if (![27, 28].includes(this.recovery)) {
            throw Error(`Signature: invalid signature. V must be 27 or 28. actual: ${this.recovery}`);
        }
        if (!this.isCompatibleEip2()) {
            const s = new BN(this.s);
            const recovery = Buffer.alloc(1);
            recovery.writeUInt8((this.v % 2) + 27, 0);
            this.buffer = Buffer.concat([
                this.r,
                secp256k1N.sub(s).toBuffer("be", 32),
                recovery,
            ]);
        }
    }
    static fromRSV(r, s, v) {
        const recovery = Buffer.alloc(1);
        recovery.writeUInt8(v, 0);
        return new Signature(Buffer.concat([r, s, recovery]));
    }
    static fromDigest(digest, address, r, s) {
        const candidate = [...new Array(2).keys()].filter((v) => {
            const publicKey = recover(digest, r, s, v);
            return address.equals(Address.fromPublicKey(publicKey));
        });
        if (candidate.length === 1) {
            const v = candidate[0] + 27;
            return Signature.fromRSV(r, s, v);
        }
        throw new Error(`Signature: failed to solve V.`);
    }
    get r() {
        return this.buffer.slice(0, 32);
    }
    get s() {
        return this.buffer.slice(32, 64);
    }
    get v() {
        return this.buffer.readUInt8(64);
    }
    get recovery() {
        return this.v;
    }
    isCompatibleEip2() {
        const s = new BN(this.s);
        return secp256k1halfN.cmp(s) > 0;
    }
    toString() {
        return this.buffer.toString("hex");
    }
}
