import { KMSClient, SignCommand, GetPublicKeyCommand, } from "@aws-sdk/client-kms";
import { Signature } from "../signature";
import { parseSignature, parsePublicKey } from "../asn1-parser";
import { Address } from "../address";
export class KmsSigner {
    constructor(keyId, config) {
        this.keyId = keyId;
        this.client = new KMSClient(config !== null && config !== void 0 ? config : {});
    }
    async sign(digest) {
        const asn1Signature = await this._sign(digest);
        const address = await this.getAddress();
        const { r, s } = parseSignature(asn1Signature);
        return Signature.fromDigest(digest, address, r, s);
    }
    async getAddress() {
        const asn1PublicKey = await this.getPublicKey();
        const publicKey = parsePublicKey(asn1PublicKey);
        return Address.fromPublicKey(publicKey);
    }
    async getPublicKey() {
        const command = new GetPublicKeyCommand({ KeyId: this.keyId });
        const response = await this.client.send(command);
        if (!response.PublicKey) {
            throw new TypeError("PublicKey is undefined");
        }
        return Buffer.from(response.PublicKey);
    }
    async _sign(digest) {
        const command = new SignCommand({
            KeyId: this.keyId,
            Message: digest,
            MessageType: "DIGEST",
            SigningAlgorithm: "ECDSA_SHA_256",
        });
        const response = await this.client.send(command);
        if (!response.Signature) {
            throw new TypeError("Signature is undefined");
        }
        return Buffer.from(response.Signature);
    }
}
